@@ -190,10 +190,10 @@ static void load_file (char *);
 char toprint[256];		       /* LUT: printable versions of chars */
 char hex[256][3];		       /* LUT: binary to hex, 1 byte */
 
-char message[80];
+char message[512];
 
-char decstatus[] = "%s TWEAK "VER": %-18.18s %s posn=%-10"OFF"d size=%-10"OFF"d";
-char hexstatus[] = "%s TWEAK "VER": %-18.18s %s posn=0x%-8"OFF"X size=0x%-8"OFF"X";
+char decstatus[] = "%s TWEAK: %-18.18s %s posn=%-10"OFF"d size=%-10"OFF"d";
+char hexstatus[] = "%s TWEAK: %-18.18s %s posn=0x%-8"OFF"X size=0x%-8"OFF"X";
 char *statfmt = hexstatus;
 
 char last_char;
@@ -219,6 +219,14 @@ fileoffset_t file_size = 0, top_pos = 0, cur_pos = 0, mark_point = 0;
 
 int scrlines;
 
+void usage(FILE *fp)
+{
+    fprintf(fp,
+            "usage: %s [-f] [-l] [-e] filename\n"
+            "    or %s -D to write default tweak.rc to stdout\n",
+            pname, pname);
+}
+
 /*
  * Main program
  */
@@ -230,17 +238,26 @@ int main(int argc, char **argv) {
      */
     pname = *argv;		       /* program name */
     if (argc < 2) {
-	fprintf(stderr,
-		"usage: %s [-f] [-l] [-e] filename\n"
-		"    or %s -D to write default tweak.rc to stdout\n",
-		pname, pname);
-	return 0;
+        usage(stderr);
+	return 1;
     }
 
     while (--argc > 0) {
 	char c, *p = *++argv, *value;
 
-	if (*p == '-') {
+        if (*p == '-' && p[1] == '-') {
+            if (!strcmp(p, "--version")) {
+                printf("%s version %s\n", pname, VER);
+                return 0;
+            } else if (!strcmp(p, "--help")) {
+                usage(stdout);
+                return 0;
+            } else {
+                fprintf(stderr, "%s: unrecognised long option `%s'\n",
+                        pname, p);
+                return 1;
+            }
+        } else if (*p == '-') {
 	    p++;
 	    while (*p) switch (c = *p++) {
 	      case 'o': case 'O':
@@ -318,7 +335,8 @@ int main(int argc, char **argv) {
 void fix_offset(void) {
     if (3*width+11 > display_cols) {
 	width = (display_cols-11) / 3;
-	sprintf (message, "Width reduced to %"OFF"d to fit on the screen", width);
+	snprintf (message, sizeof(message),
+		  "Width reduced to %"OFF"d to fit on the screen", width);
     }
     if (4*width+14 > display_cols) {
 	ascii_enabled = FALSE;
@@ -384,13 +402,15 @@ static void load_file (char *fname) {
 	    }
 	    fclose (fp);
 	    assert(file_size == buf_length(filedata));
-	    sprintf(message, "loaded %s (size %"OFF"d == 0x%"OFF"X).",
-		    fname, file_size, file_size);
+	    snprintf(message, sizeof(message),
+		     "loaded %s (size %"OFF"d == 0x%"OFF"X).",
+		     fname, file_size, file_size);
 	} else {
 	    filedata = buf_new_from_file(fp);
 	    file_size = buf_length(filedata);
-	    sprintf(message, "opened %s (size %"OFF"d == 0x%"OFF"X).",
-		    fname, file_size, file_size);
+	    snprintf(message, sizeof(message),
+		     "opened %s (size %"OFF"d == 0x%"OFF"X).",
+		     fname, file_size, file_size);
 	}
 	new_file = FALSE;
     } else {
@@ -400,7 +420,7 @@ static void load_file (char *fname) {
 	    exit (1);
 	}
 	filedata = buf_new_empty();
-	sprintf(message, "New file %s.", fname);
+	snprintf(message, sizeof(message), "New file %s.", fname);
 	new_file = TRUE;
     }
 }
@@ -469,7 +489,7 @@ int backup_file (void) {
 }
 
 static unsigned char *scrbuf = NULL;
-static int scrbuflines = 0;
+static int scrbufsize = 0;
 
 /*
  * Draw the screen, for normal usage.
@@ -484,16 +504,15 @@ void draw_scr (void) {
     char *linebuf;
 
     scrlines = display_rows - 2;
-    if (scrlines > scrbuflines) {
-	scrbuf = (scrbuf ?
-		  realloc(scrbuf, scrlines*width) :
-		  malloc(scrlines*width));
+    scrsize = scrlines * width;
+    if (scrsize > scrbufsize) {
+	scrbuf = (scrbuf ? realloc(scrbuf, scrsize) : malloc(scrsize));
 	if (!scrbuf) {
 	    done();
 	    fprintf(stderr, "%s: out of memory!\n", pname);
 	    exit (2);
 	}
-	scrbuflines = scrlines;
+	scrbufsize = scrsize;
     }
 
     linebuf = malloc(width*4+20);
@@ -509,7 +528,8 @@ void draw_scr (void) {
 	scroff = width - offset;
     else
 	scroff = 0;
-    scrsize = scrlines * width - scroff;
+
+    scrsize -= scroff;
     if (scrsize > file_size - top_pos)
 	scrsize = file_size - top_pos;
 
@@ -691,7 +711,7 @@ int get_str (char *prompt, char *buf, int highlight) {
 	} else if (c == 27 || c == 7) {
 	    display_beep();
 	    display_post_error();
-	    strcpy (message, "User Break!");
+	    snprintf(message, sizeof(message), "User Break!");
 	    return FALSE;
 	}
 
@@ -760,7 +780,7 @@ void suspend(void) {
     init();
 #else
     display_beep();
-    strcpy(message, "Suspend function not yet implemented.");
+    snprintf(message, sizeof(message), "Suspend function not yet implemented.");
 #endif
 }
 
